@@ -16,7 +16,7 @@ import logging
 import uuid
 from datetime import datetime
 from functools import reduce
-from typing import Dict, List
+from typing import Dict, List, Tuple
 
 from reana_commons.config import (
     MQ_MAX_PRIORITY,
@@ -32,9 +32,10 @@ from sqlalchemy import (
     Column,
     DateTime,
     Enum,
-    Float,
     ForeignKey,
+    Index,
     Integer,
+    MetaData,
     String,
     Text,
     UniqueConstraint,
@@ -54,19 +55,31 @@ from reana_db.config import (
     DB_SECRET_KEY,
     DEFAULT_QUOTA_LIMITS,
     DEFAULT_QUOTA_RESOURCES,
-    LIMIT_RESTARTS,
     WORKFLOW_TERMINATION_QUOTA_UPDATE_POLICY,
 )
 from reana_db.utils import (
     build_workspace_path,
     store_workflow_disk_quota,
+    split_run_number,
+    update_users_cpu_quota,
     update_users_disk_quota,
     update_workflow_cpu_quota,
     update_workspace_retention_rules,
 )
 
 
-Base = declarative_base()
+convention = {
+    "ix": "ix_%(column_0_label)s",
+    "uq": "uq_%(table_name)s_%(column_0_name)s",
+    "ck": "ck_%(table_name)s_%(constraint_name)s",
+    "fk": "fk_%(table_name)s_%(column_0_name)s_%(referred_table_name)s",
+    "pk": "pk_%(table_name)s",
+}
+"""Constraint naming convention."""
+
+metadata_obj = MetaData(naming_convention=convention)
+
+Base = declarative_base(metadata=metadata_obj)
 
 
 def generate_uuid():
@@ -278,7 +291,7 @@ class User(Base, Timestamp, QuotaBase):
                 Workflow.status == RunStatus.running,
             )
         ).count()
-        if running_count > max_concurrent_workflows:
+        if running_count >= max_concurrent_workflows:
             return 0.1
         # we reduce the 10% (* 0.9) to avoid getting a 0 multiplier factor when
         # `running_count == `max_concurrent_workflows`, thus taking into
@@ -419,7 +432,7 @@ class InteractiveSession(Base, Timestamp, QuotaBase):
     """Interactive Session table."""
 
     __tablename__ = "interactive_session"
-    id_ = Column(UUIDType, primary_key=True, unique=True, default=generate_uuid)
+    id_ = Column(UUIDType, primary_key=True, default=generate_uuid)
     name = Column(String(255))
     path = Column(Text)  # path to access the interactive session
     status = Column(Enum(RunStatus), nullable=False, default=RunStatus.created)
@@ -431,7 +444,7 @@ class InteractiveSession(Base, Timestamp, QuotaBase):
     )
 
     __table_args__ = (
-        UniqueConstraint("name", "path", name="_interactive_session_uc"),
+        UniqueConstraint("name", "path"),
         {"schema": "__reana"},
     )
 
@@ -447,7 +460,7 @@ class Workflow(Base, Timestamp, QuotaBase):
 
     id_ = Column(UUIDType, primary_key=True)
     name = Column(String(255))
-    status = Column(Enum(RunStatus), default=RunStatus.created)
+    status = Column(Enum(RunStatus), default=RunStatus.created, index=True)
     owner_id = Column(UUIDType, ForeignKey("__reana.user_.id_"))
     reana_specification = Column(JSONType)
     input_parameters = Column(JSONType)
@@ -458,7 +471,8 @@ class Workflow(Base, Timestamp, QuotaBase):
     run_started_at = Column(DateTime)
     run_finished_at = Column(DateTime)
     run_stopped_at = Column(DateTime)
-    _run_number = Column("run_number", Float)
+    run_number_major = Column(Integer)
+    run_number_minor = Column(Integer, default=0)
     job_progress = Column(JSONType, default=dict)
     workspace_path = Column(String)
     restart = Column(Boolean, default=False)
@@ -486,7 +500,10 @@ class Workflow(Base, Timestamp, QuotaBase):
 
     __table_args__ = (
         UniqueConstraint(
-            "name", "owner_id", "run_number", name="_user_workflow_run_uc"
+            "owner_id",
+            "name",
+            "run_number_major",
+            "run_number_minor",
         ),
         {"schema": "__reana"},
     )
@@ -526,7 +543,9 @@ class Workflow(Base, Timestamp, QuotaBase):
         self.git_repo = git_repo
         self.git_provider = git_provider
         self.restart = restart
-        self._run_number = self.assign_run_number(run_number)
+        self.run_number_major, self.run_number_minor = self.get_new_run_number(
+            run_number
+        )
         self.workspace_path = workspace_path or build_workspace_path(
             self.owner_id, self.id_
         )
@@ -536,54 +555,66 @@ class Workflow(Base, Timestamp, QuotaBase):
         """Workflow string representation."""
         return "<Workflow %r>" % self.id_
 
-    @hybrid_property
-    def run_number(self):
-        """Property of run_number."""
-        if self._run_number.is_integer():
-            return int(self._run_number)
-        return self._run_number
+    @property
+    def run_number(self) -> str:
+        """Get workflow run number."""
+        if self.run_number_minor != 0:
+            return f"{self.run_number_major}.{self.run_number_minor}"
+        return str(self.run_number_major)
 
-    @run_number.expression
-    def run_number(cls):
-        return func.abs(cls._run_number)
-
-    def assign_run_number(self, run_number):
-        """Assing run number."""
+    def _get_last_workflow(self, run_number):
+        """Fetch the last workflow restart given a certain run number."""
         from .database import Session
 
         if run_number:
+            run_number_major, run_number_minor = split_run_number(run_number)
             last_workflow = (
                 Session.query(Workflow)
                 .filter(
                     Workflow.name == self.name,
-                    Workflow.run_number >= int(run_number),
-                    Workflow.run_number < int(run_number) + 1,
+                    Workflow.run_number_major == run_number_major,
                     Workflow.owner_id == self.owner_id,
                 )
-                .order_by(Workflow.run_number.desc())
+                .order_by(
+                    Workflow.run_number_major.desc(), Workflow.run_number_minor.desc()
+                )
                 .first()
             )
         else:
             last_workflow = (
                 Session.query(Workflow)
                 .filter_by(name=self.name, restart=False, owner_id=self.owner_id)
-                .order_by(Workflow.run_number.desc())
+                .order_by(
+                    Workflow.run_number_major.desc(), Workflow.run_number_minor.desc()
+                )
                 .first()
             )
-        if last_workflow and self.restart:
-            # FIXME: remove the limit of nine restarts when we fix the way in which
-            # we save `run_number` in the DB
-            num_restarts = round(last_workflow.run_number * 10) % 10
-            if num_restarts == LIMIT_RESTARTS:
+        return last_workflow
+
+    def get_new_run_number(self, run_number) -> Tuple[int, int]:
+        """Return the major and minor run numbers for a new workflow.
+
+        Return a tuple where the first element is the major run number and the
+        second element is the minor run number.
+        """
+        last_workflow = self._get_last_workflow(run_number)
+
+        if not last_workflow:
+            if self.restart:
                 raise REANAValidationError(
-                    f"Cannot restart a workflow more than {LIMIT_RESTARTS} times"
+                    "Cannot restart a workflow that has not been run before."
                 )
-            return round(last_workflow.run_number + 0.1, 1)
+            return 1, 0  # First workflow run
+
         else:
-            if not last_workflow:
-                return 1
+            if not self.restart:
+                run_number_major = last_workflow.run_number_major + 1
+                run_number_minor = 0
             else:
-                return last_workflow.run_number + 1
+                run_number_major = last_workflow.run_number_major
+                run_number_minor = last_workflow.run_number_minor + 1
+
+            return run_number_major, run_number_minor
 
     def get_input_parameters(self):
         """Return workflow parameters."""
@@ -603,7 +634,7 @@ class Workflow(Base, Timestamp, QuotaBase):
 
     def get_full_workflow_name(self):
         """Return full workflow name including run number."""
-        return "{}.{}".format(self.name, str(self.run_number))
+        return "{}.{}".format(self.name, self.run_number)
 
     def get_workspace_disk_usage(self, summarize=False, search=None):
         """Retrieve disk usage information of a workspace."""
@@ -642,15 +673,13 @@ class Workflow(Base, Timestamp, QuotaBase):
         """Get all the restarts of this workflow, including the original workflow.
 
         Returns all the restarts of this workflow, that is all the workflows that have
-        the same name and the same run number (up to the dot). This includes the
+        the same name and the same major run number. This includes the
         original workflow, as well as all the following restarts.
         """
-        run_number = int(self.run_number)
         restarts = Workflow.query.filter(
             Workflow.name == self.name,
             Workflow.owner_id == self.owner_id,
-            Workflow.run_number >= run_number,
-            Workflow.run_number < run_number + 1,
+            Workflow.run_number_major == self.run_number_major,
         )
         return restarts
 
@@ -757,8 +786,8 @@ def workflow_status_change_listener(workflow, new_status, old_status, initiator)
             return
 
         try:
-            update_users_disk_quota(user=workflow.owner)
             store_workflow_disk_quota(workflow)
+            update_users_disk_quota(user=workflow.owner)
         except Exception as e:
             logging.error(f"Failed to update disk quota: \n{e}\nContinuing...")
 
@@ -768,6 +797,7 @@ def workflow_status_change_listener(workflow, new_status, old_status, initiator)
 
         try:
             update_workflow_cpu_quota(workflow=workflow)
+            update_users_cpu_quota(user=workflow.owner)
         except Exception as e:
             logging.error(f"Failed to update cpu quota: \n{e}\nContinuing...")
 
@@ -791,7 +821,6 @@ class Job(Base, Timestamp):
     """Job table."""
 
     __tablename__ = "job"
-    __table_args__ = {"schema": "__reana"}
 
     id_ = Column(UUIDType, primary_key=True, default=generate_uuid)
     backend_job_id = Column(String(256))
@@ -810,6 +839,11 @@ class Job(Base, Timestamp):
     prettified_cmd = Column(JSONType)
     job_name = Column(Text)
 
+    __table_args__ = (
+        Index(None, "workflow_uuid", "created"),
+        {"schema": "__reana"},
+    )
+
 
 @event.listens_for(Job.status, "set")
 def job_status_change_listener(job, new_status, old_status, initiator):
@@ -943,9 +977,7 @@ class WorkspaceRetentionRule(Base):
     )
 
     __table_args__ = (
-        UniqueConstraint(
-            "workflow_id", "workspace_files", name="_workspace_retention_rule_uc"
-        ),
+        UniqueConstraint("workflow_id", "workspace_files"),
         {"schema": "__reana"},
     )
 
@@ -1026,9 +1058,11 @@ class ResourceUnit(enum.Enum):
 
         converted_value = round(bytes_ / math.pow(k, unit_index), digits)
         return "{converted_value} {converted_unit}".format(
-            converted_value=int(converted_value)
-            if converted_value.is_integer()
-            else converted_value,
+            converted_value=(
+                int(converted_value)
+                if converted_value.is_integer()
+                else converted_value
+            ),
             converted_unit=units[unit_index],
         )
 
