@@ -30,6 +30,9 @@ from reana_db.models import (
 from reana_db.utils import (
     get_default_quota_resource,
     update_users_cpu_quota,
+    update_workflows_cpu_quota,
+    update_workflows_disk_quota,
+    update_users_disk_quota,
     update_workspace_retention_rules,
 )
 
@@ -48,7 +51,9 @@ def test_workflow_run_number_assignment(db, session, new_user):
     )
     session.add(first_workflow)
     session.commit()
-    assert first_workflow.run_number == 1
+    assert first_workflow.run_number == "1"
+    assert first_workflow.run_number_major == 1
+    assert first_workflow.run_number_minor == 0
     second_workflow = Workflow(
         id_=str(uuid4()),
         name=workflow_name,
@@ -59,7 +64,9 @@ def test_workflow_run_number_assignment(db, session, new_user):
     )
     session.add(second_workflow)
     session.commit()
-    assert second_workflow.run_number == 2
+    assert second_workflow.run_number == "2"
+    assert second_workflow.run_number_major == 2
+    assert second_workflow.run_number_minor == 0
     first_workflow_restart = Workflow(
         id_=str(uuid4()),
         name=workflow_name,
@@ -72,7 +79,9 @@ def test_workflow_run_number_assignment(db, session, new_user):
     )
     session.add(first_workflow_restart)
     session.commit()
-    assert first_workflow_restart.run_number == 1.1
+    assert first_workflow_restart.run_number == "1.1"
+    assert first_workflow_restart.run_number_major == 1
+    assert first_workflow_restart.run_number_minor == 1
     first_workflow_second_restart = Workflow(
         id_=str(uuid4()),
         name=workflow_name,
@@ -85,7 +94,9 @@ def test_workflow_run_number_assignment(db, session, new_user):
     )
     session.add(first_workflow_second_restart)
     session.commit()
-    assert first_workflow_second_restart.run_number == 1.2
+    assert first_workflow_second_restart.run_number == "1.2"
+    assert first_workflow_second_restart.run_number_major == 1
+    assert first_workflow_second_restart.run_number_minor == 2
 
 
 def test_workflow_retention_rules(db, session, new_user):
@@ -274,6 +285,10 @@ def test_access_token(db, session, new_user):
     "reana_db.models.WORKFLOW_TERMINATION_QUOTA_UPDATE_POLICY",
     [ResourceType.cpu.name, ResourceType.disk.name],
 )
+@mock.patch(
+    "reana_db.utils.WORKFLOW_TERMINATION_QUOTA_UPDATE_POLICY",
+    [ResourceType.cpu.name, ResourceType.disk.name],
+)
 def test_workflow_cpu_quota_usage_update(db, session, run_workflow):
     """Test quota usage update once workflow is finished/stopped/failed."""
     time_elapsed_seconds = 0.5
@@ -310,7 +325,6 @@ def test_workflow_termination_user_quota_usage(
         "reana_db.utils.WORKFLOW_TERMINATION_QUOTA_UPDATE_POLICY",
         workflow_termination_quota_update_policy,
     ):
-
         time_elapsed_seconds = 0.5
         num_workflows = 2
         for n in range(num_workflows):
@@ -327,7 +341,7 @@ def test_workflow_termination_user_quota_usage(
             quota_usage["cpu"]["usage"]["raw"] == 0
 
         if ResourceType.disk.name in workflow_termination_quota_update_policy:
-            assert quota_usage["disk"]["usage"]["raw"] == 128
+            assert quota_usage["disk"]["usage"]["raw"] == 128 * num_workflows
         else:
             quota_usage["disk"]["usage"]["raw"] == 0
 
@@ -353,6 +367,9 @@ def test_all_users_cpu_quota_usage_update(
     with mock.patch(
         "reana_db.models.WORKFLOW_TERMINATION_QUOTA_UPDATE_POLICY",
         workflow_termination_quota_update_policy,
+    ), mock.patch(
+        "reana_db.utils.WORKFLOW_TERMINATION_QUOTA_UPDATE_POLICY",
+        workflow_termination_quota_update_policy,
     ), mock.patch(
         "reana_db.utils.PERIODIC_RESOURCE_QUOTA_UPDATE_POLICY",
         periodic_update,
@@ -368,6 +385,7 @@ def test_all_users_cpu_quota_usage_update(
         else:
             assert quota_cpu_usage == 0
 
+        update_workflows_cpu_quota()
         update_users_cpu_quota()
 
         quota_cpu_usage = new_user.get_quota_usage()["cpu"]["usage"]["raw"]
@@ -377,6 +395,60 @@ def test_all_users_cpu_quota_usage_update(
             assert quota_cpu_usage == 0
 
 
+@pytest.mark.parametrize(
+    "workflow_termination_quota_update_policy, periodic_update",
+    [
+        ([], True),
+        ([], False),
+        ([ResourceType.disk.name], False),
+        ([ResourceType.disk.name], True),
+    ],
+)
+def test_all_users_disk_quota_usage_update(
+    db,
+    session,
+    new_user,
+    run_workflow,
+    workflow_termination_quota_update_policy,
+    periodic_update,
+):
+    """Test disk periodic update cronjob functionality."""
+    num_workflows = 2
+    dir_size = 128
+    with mock.patch(
+        "reana_db.models.WORKFLOW_TERMINATION_QUOTA_UPDATE_POLICY",
+        workflow_termination_quota_update_policy,
+    ), mock.patch(
+        "reana_db.utils.WORKFLOW_TERMINATION_QUOTA_UPDATE_POLICY",
+        workflow_termination_quota_update_policy,
+    ), mock.patch(
+        "reana_db.utils.PERIODIC_RESOURCE_QUOTA_UPDATE_POLICY",
+        periodic_update,
+    ), mock.patch(
+        "reana_db.utils.get_disk_usage", return_value=[{"size": {"raw": str(dir_size)}}]
+    ):
+        workflows = [run_workflow() for n in range(num_workflows)]
+
+        quota_disk_usage = new_user.get_quota_usage()["disk"]["usage"]["raw"]
+        if workflow_termination_quota_update_policy:
+            for wf in workflows:
+                assert wf.resources[0].quota_used == dir_size
+            assert quota_disk_usage == dir_size * num_workflows
+        else:
+            assert quota_disk_usage == 0
+
+        update_workflows_disk_quota()
+        update_users_disk_quota()
+
+        quota_disk_usage = new_user.get_quota_usage()["disk"]["usage"]["raw"]
+        if workflow_termination_quota_update_policy or periodic_update:
+            for wf in workflows:
+                assert wf.resources[0].quota_used == dir_size
+            assert quota_disk_usage == dir_size * num_workflows
+        else:
+            assert quota_disk_usage == 0
+
+
 @pytest.mark.parametrize(
     "unit, value, human_readable_string",
     [
